using System;

using Keyoti.RapidSpell;
using System.ComponentModel;

namespace ComponaSyntaxBox_RSDT.NET
{
	/// <summary>
	/// Wraps SyntaxBox in an interface that works with RapidSpell Desktop .NET
	/// </summary>
	[ToolboxItem(true), DesignTimeVisible(true), Designer(typeof(System.ComponentModel.Design.ComponentDesigner)),
	DefaultProperty("TextControl")]
	public class PlainSyntaxBoxAdapter : Keyoti.RapidSpell.ISpellCheckableTextComponent
	{
		Compona.Windows.Forms.SyntaxBoxControl textBox;
		public Compona.Windows.Forms.SyntaxBoxControl TextControl
		{
			get{return textBox;}
			set{textBox=value;
				if(textBox!=null)
					textBox.Enter+= new EventHandler(TBGotFocus);
			}
		}


		///<summary>Gets or sets a value indicating whether the selected text in the text box control remains highlighted when the control loses focus.</summary>
		[Browsable(false), DesignerSerializationVisibility(DesignerSerializationVisibility.Hidden)]
		public bool HideSelection
		{
			get
			{ 
				if(textBox==null) throw new InvalidOperationException("SyntaxBoxAdapter.TextControl property not set to (non-null) textBox instance (set TextControl property before starting spell check).");
				return true;
			}
			set
			{ 
				if(textBox==null) throw new InvalidOperationException("SyntaxBoxAdapter.TextControl property not set to (non-null) textBox instance (set TextControl property before starting spell check).");
				//can't do anything, not supported
			}
		}


		///<summary>Gets or sets the current text in the text box.</summary>
		[Browsable(false), DesignerSerializationVisibility(DesignerSerializationVisibility.Hidden)]
		public string Text
		{
			get
			{ 
				if(textBox==null) throw new InvalidOperationException("SyntaxBoxAdapter.TextControl property not set to (non-null) textBox instance (set TextControl property before starting spell check).");			
				return textBox.Document.Text; }
			set
			{ 
				if(textBox==null) throw new InvalidOperationException("SyntaxBoxAdapter.TextControl property not set to (non-null) textBox instance (set TextControl property before starting spell check).");			
				textBox.Document.Text = value;
				textBox.Refresh ();
			}
		}


		///<summary>Gets or sets a value indicating the currently selected text in the control.</summary>
		[Browsable(false), DesignerSerializationVisibility(DesignerSerializationVisibility.Hidden)]
		public String SelectedText
		{
			get
			{
				if(textBox==null) throw new InvalidOperationException("SyntaxBoxAdapter.TextControl property not set to (non-null) textBox instance (set TextControl property before starting spell check).");
				return textBox.Selection.Text;
			}
			set
			{
				if(textBox==null) throw new InvalidOperationException("SyntaxBoxAdapter.TextControl property not set to (non-null) textBox instance (set TextControl property before starting spell check).");
				textBox.Selection.Text = value;
				textBox.Refresh ();
			}
		}

		///<summary>Gets or sets the starting point of text selected in the text box.</summary>
		[Browsable(false), DesignerSerializationVisibility(DesignerSerializationVisibility.Hidden)]
		public int SelectionStart
		{
			get
			{ 
				if(textBox==null) throw new InvalidOperationException("SyntaxBoxAdapter.TextControl property not set to (non-null) textBox instance (set TextControl property before starting spell check).");
				return textBox.Selection.SelStart;
			}
			set
			{ 
				if(textBox==null) throw new InvalidOperationException("SyntaxBoxAdapter.TextControl property not set to (non-null) textBox instance (set TextControl property before starting spell check).");
				textBox.Selection.ClearSelection();
				textBox.Selection.SelStart = value;
				textBox.Selection.SelLength=0;

				//ensure the rows is visible (unfold collapsed sections)
				Compona.SourceCode.Row row=textBox.Document[textBox.Selection.Bounds.FirstRow];
				if (row == null) throw new InvalidOperationException ("Selection is out of bounds");
				row.EnsureVisible ();
				//place caret at selectionstart
				textBox.Caret.SetPos (new Compona.SourceCode.TextPoint(this.textBox.Selection.Bounds.FirstColumn,this.textBox.Selection.Bounds.FirstRow));
				textBox.Refresh ();
			}
		}

		///<summary>Gets or sets the number of characters selected in the text box.</summary>
		[Browsable(false), DesignerSerializationVisibility(DesignerSerializationVisibility.Hidden)]
		public int SelectionLength
		{
			get
			{
				if(textBox==null) throw new InvalidOperationException("SyntaxBoxAdapter.TextControl property not set to (non-null) textBox instance (set TextControl property before starting spell check).");
				return textBox.Selection.SelLength;
			}
			set
			{ 
				if(textBox==null) throw new InvalidOperationException("SyntaxBoxAdapter.TextControl property not set to (non-null) textBox instance (set TextControl property before starting spell check).");
				textBox.Selection.SelLength = value;
				textBox.Refresh ();
			}
		}

		///<summary>Scrolls to caret.</summary>
		[Browsable(false)]
		public void ScrollToCaret()
		{
			textBox.ScrollIntoView();
			textBox.Refresh ();
		}
		

		///<summary>Occurs when the control receives focus.</summary>
		public event EventHandler GotFocus;

		///<summary>Called when the control receives focus.</summary>
		[Browsable(false)]
		public void TBGotFocus(object sender, EventArgs e)
		{
			if(GotFocus!=null) GotFocus(this, e);
		}

		
	}
}
